<?php

/*****************************************************************************
 * 
 * サンプルのため、冗長なコメントになっています。
 * 
 ****************************************************************************/

/**

以下の DDL でテーブルを作成してください。

CREATE TABLE `test_table` (
    `test_table_id` int  NOT NULL  auto_increment ,
    `name` varchar(255)  NOT NULL   ,
    `email` varchar(255)  NOT NULL   ,
    `os` int  NOT NULL   ,
    `within` int  NOT NULL   ,
    `browser` varchar(255)  NOT NULL   ,
    `message` text  NOT NULL   ,
    `reg_datetime` datetime  NOT NULL   ,
    `upd_datetime` datetime  NULL   ,
    PRIMARY KEY (test_table_id)
) TYPE=InnoDB CHARACTER SET UTF8;

*/

/**
 * サンプルのため、定数定義をこのファイル内に記述しています。
 * 本来は、Lambda/apps/<APP_DIR>/app.conf.php に定義することが推奨されます。
 */
// ドライラン： 更新系SQLを実行する = 0 | 更新系SQLを実行しない = 1
//
//   ※ サンプルのため、この定数を定義しています。本来は不要です。
//      また、ロジックとビューでこの定数を参照しているステートメントも不要になります。
define('DATABASE_DRYRUN', 1);

// Rows Per Page: 1ページに収める行数
define('RPP', 5);

/**
 * リクエストが「http://example.com/database」のように、/ (スラッシュ)で
 * 終わっていない場合、/ (スラッシュ)付きURLにリダイレクトする。
 *
 *   ビュー(index.view.php)の <form action="" ... > にて、
 *   
 *     <form action="<?php echo APP_ROOT ?>/database/" ... >
 *   
 *   上記のように、モジュールグループを直接記述すれば、
 *   このステートメントは不要になります。
 *   ※ 本来は、上記のように直接記述することが推奨されます。
 *   
 *   本モジュールはサンプルのため、環境に依存した記述を減らし、
 *   設置の誤りを防ぐ意味で、下記のように <form action> の値を動的に
 *   生成している都合上、このようなリダイレクト処理が必要になっています。
 *   
 *   <form action="<?php echo $lmd->httpRequestDir() ?>/">
 */
if (substr($lmd->httpRequestUri(), -9) === '/database') {
    $lmd->redirect($lmd->httpRequestUri().'/');
    exit;
}

/**
 * ジャバスクリプトライブラリのロード
 * 
 *   ジャバスクリプトライブラリは、Lambda/libs/Javascript.php にあります。
 * 
 * [tips]
 * よく使うライブラリは、Lambda/apps/<APP_DIR>/app.init.php で事前にロードしておくとよい。
 */
Factory::import('Javascript');

/**
 * DAOライブラリをロードし、インスタンス化する
 */
$dao = Factory::load('DB/DAO_test_table')->getInstance();



/*****************************************************************************
 * 以下、【確認】【登録】【修正】のいずれかのボタンが押された時の処理
 */

/**
 * POSTパラメーターを配列に格納。
 *
 *   こうしておくと、後続処理の setResponse や invokeValidator に値を渡す際に
 *   簡素に記述できます。
 */
$p = array(
    'name'    => $lmd->httpPost('name'),
    'email'   => $lmd->httpPost('email'),
    'os'      => $lmd->httpPost('os'),
    'within'  => $lmd->httpPost('within'),
    'browser' => $lmd->httpPost('browser', array()),
    'message' => $lmd->httpPost('message'),
);

/**
 * 主キー：test_table_id の取得
 */
$test_table_id = $lmd->httpPost('test_table_id');

/**
 * レスポンスをアサイン
 */
$lmd->setResponse($p);

if ($lmd->hasHttpPost('confirm') || $lmd->hasHttpPost('commit') || $lmd->hasHttpPost('edit')) {
    /**
     * 【確認】【登録】【修正】のいずれかのボタンが押された。
     * 
     *   バリデーターを実行して、入力値の検証を行います。
     *   ※ バリデーター定義は、Lambda/validators/mailform.php にあります。
     */
    $lmd->invokeValidator('mailform', $p);
}

if ($lmd->hasValidatorError()) {
    /**
     * 入力エラーが発見された。
     */

    if ($lmd->hasHttpPost('edit')) {
        /**
         * 遷移元は「修正画面」だったので、
         *   主キーである「test_table_id」をレスポンスにセットし、
         *   「修正画面」に遷移させる。
         */
        $lmd->setResponse(array('test_table_id' => $test_table_id));
        return $lmd->setPrep('database.edit');
    } else {
        /**
         * 「入力画面」に遷移させる。
         */
        return $lmd->setPrep('database.index');
    }
}

if ($lmd->hasHttpPost('confirm') && ! $lmd->hasValidatorError()) {
    /**
     * 【確認】ボタンが押され、入力エラーが無い。
     *
     *   ⇒ 「確認画面」に遷移させる。
     */
    return $lmd->setPrep('database.confirm');
}

if (($lmd->hasHttpPost('commit') || $lmd->hasHttpPost('edit')) && ! $lmd->hasValidatorError()) {

    /**
     * 【登録】または【修正】ボタンが押され、入力エラーが無い。
     */

    /**
     * ブラウザ： カンマ区切りに変換
     */
    $p['browser'] = implode(',', $p['browser']);

    $result = false;
    if ($lmd->hasHttpPost('commit')) {
        /**
         * 【登録】ボタンが押されていた。
         * 
         *    ⇒ データベースに登録(insert)する。
         */

        /**
         * 登録実行(insert)
         */
        if ( ! DATABASE_DRYRUN) $result = $dao->insert($p);
    }

    if ($lmd->hasHttpPost('edit')) {
        /**
         * 【修正】ボタンが押されていた。
         * 
         *    ⇒ データベースを更新(update)する。
         */

        /**
         * 更新実行(update)
         */
        if ( ! DATABASE_DRYRUN) $result = $dao->update($test_table_id, $p);
    }

    if (DATABASE_DRYRUN) {
        /**
         * ドライラン
         */
        Javascript::redirect($lmd->httpRequestDir().'/', 'サンプルのため、インサートしません');
    } elseif ($result) {
        /**
         * 登録/更新 成功
         * 
         *    リロード操作での二重送信を防止するため、リダイレクトさせます。
         */
        $lmd->redirect($lmd->httpRequestDir().'/');
        exit;
    } else {
        /**
         * 登録/更新 失敗
         * 
         *    遷移元に戻します。
         */
        Javascript::backHistory('登録に失敗しました', '#ffa07a');
    }
}



/*****************************************************************************
 * 以下、すでに登録されているデータの取得とページャ処理
 */

/**
 * ページ番号初期化
 */
$page = $lmd->httpGet('page', 1, '/^[0-9]+$/');

/**
 * offset と limit を定義
 */
$offset = ($page - 1) * RPP;
$limit  = RPP;

/**
 * DBアクセスして、行を取得
 */
$res = $dao->fetchAll($offset, $limit);

/**
 * 行の総数を取得
 */
$total_rows = $dao->foundRows();

/**
 * 取得したデータをレスポンスにアサイン
 */
$lmd->setResponse(array(
    'res' => $res,
));

/**
 * ページャーライブラリをロードして、ページャーをアサイン
 */
Factory::import('Pager');
$lmd->setResponse(array(
    'pager' => Pager::GETParam($page, $limit, $total_rows)
), Lambda::INSANITATION);



/*****************************************************************************
 * ここまで来たら、暗黙のうちに database/index.prep.php に処理が移ります。
 * database/index.prep.php がない場合は、database/index.view.php に処理が移ります。
 */


